rm(list=ls())
library("reshape2")

# all time series in a single long table
x <- read.table(file=paste0("in/observations.txt"),
  sep="\t", header=TRUE,
  colClasses=c("character","POSIXct","numeric")
)

# some quick checks
print(head(x))
print(unique(x[,"variable"]))

# convenience function
plt <- function(v, x) {
  with(x[x[,"variable"]==v,], {
    plot(date, value, type="l", ylab=v)
    legend("topleft", bty="n",
      legend=paste0("n=",sum(is.finite(value))))
  })
}

# arrange plots as matrix
par(mfrow=c(2,2))
sapply(unique(x[,"variable"]), plt, x=x)
par(mfrow=c(1,1))

# transform into wide format: gaps become NA
# one could also use "pivot_wider()" from package "tidyr"
x <- reshape2::dcast(date ~ variable, data=x, value.var="value")
stopifnot(!is.unsorted(x[,"date"]))
print(head(x))
# suggested: plot P_in ~ Q_in and P_in ~ Q_in*P_in 

# fill gaps in time series by linear interpolation
for (v in c("V","P_in")) {
  i <- is.finite(x[,v])
  x[,v] <- approx(x=x[i,"date"], y=x[i,v], xout=x[,"date"])$y 
}

# drop incomplete records at head and tail 
i <- apply(x[,!names(x) %in% c("date","P_lake")], 1,
  function(z) {all(is.finite(z))})
x <- x[i,]
print(head(x))
print(tail(x))

# compute expected P_lake using the non-reactive CFSTR model
# note: a simple Euler-solution of the ODE would also work
c_lake <- function(q,v,c_in,c0,t) {
  exp(-q/v*t) * (c0- c_in) + c_in
}

x <- cbind(x, P_lake_exp=NA)       # empty result column
x[1,"P_lake_exp"] <- x[1,"P_in"]   # very 1st estimate
for (i in 1:(nrow(x)-1)) {         # add results row-wise
  x[i+1,"P_lake_exp"] <- c_lake(
    q=x[i,"Q_in"],
    v=x[i,"V"],
    c_in=x[i,"P_in"],
    c0=x[i,"P_lake_exp"],
    t=abs(as.numeric(difftime(x[i+1,"date"], x[i,"date"],
      tz="UTC", units="secs")))
  )
}

# differences in concentrations at times of in-lake measurements
x <- x[is.finite(x[,"P_lake"]),]
x <- cbind(x, diff=x[,"P_lake"]-x[,"P_lake_exp"])

# distributions by month
boxplot(diff ~ format(date,"%m"), data=x,
  xlab="Month", ylab="Observed - Expected (mg/L)")
abline(h=0, lty=3)
legend("topleft", bty="n", legend="Lake is a source")
legend("bottomleft", bty="n", legend="Lake is a sink")
